@@ -81,7 +81,8 @@ def list_queued(app, queue_names):
     return listings
 
 
-def drain_queues(app, queue_names, callbacks=None, retain=False):
+def drain_queues(app, queue_names, callbacks=None, retain=False,
+                 passive_queues=False):
     """Drain the messages from queues.
 
     :param app: The app to list queues for (affects backend, Queue type,
@@ -94,21 +95,33 @@ def drain_queues(app, queue_names, callbacks=None, retain=False):
     if callbacks is None:
         callbacks = [lambda x, y: None]
     bindings = []
-    router = app.amqp.Router()
+    router = app.amqp.Router(create_missing=True)
     for queue_name in queue_names:
         destination = router.expand_destination(queue_name)
         exchange = Exchange(destination['exchange'])
         queue = Queue(queue_name, exchange=exchange)
         bindings.append(queue)
     with app.broker_connection() as connection:
-        # The meaning of no_ack appears to be inverted.
-        # See: https://github.com/ask/kombu/issues/126
+        # The no_ack flag is misleadingly named.
+        # See: https://github.com/ask/kombu/issues/130
         consumer = Consumer(
-            connection, bindings, callbacks=callbacks, no_ack=not retain)
+            connection, bindings, callbacks=callbacks, no_ack=not retain,
+            auto_declare=not passive_queues)
+        if passive_queues:
+            # This is basically copied from kombu.Queue.declare().
+            # We can't use this method directly because queue_declare()
+            # must be called with passive=True for result queues.
+            # Otherwise, attempts to connect to the queue fails with
+            # celery.exceptions.QueueNotFound: "Queue ... is not defined
+            # in CELERY_QUEUES".
+            for queue in consumer.queues:
+                if queue.exchange:
+                    queue.exchange.declare()
+                queue.queue_declare(passive=True)
         with consumer:
             try:
                 # Timeout of 0 causes error: [Errno 11] Resource temporarily
-                # unavailable
-                connection.drain_events(timeout=.1 ** 100)
+                # unavailable.
+                connection.drain_events(timeout=0.1 ** 100)
             except timeout:
                 pass
