@@ -1,7 +1,4 @@
-BROKER_VHOST = "/"
-CELERY_RESULT_BACKEND = "amqp"
-CELERY_IMPORTS = ("lazr.jobrunner.tests.test_celerytask", )
-CELERYD_CONCURRENCY = 1
+from simple_config import *
 import os
 import oops
 CELERY_ANNOTATIONS = {
