@@ -476,3 +476,119 @@ class TestListQueues(TestCase):
     def test_no_tasks(self):
         """When no jobs are listed, the queue is shown as empty."""
         self.assertEqual([], list_queued(RunFileJob.app, [self.queue]))
+
+
+class TestClearQueues(TestCase):
+    """Tests for the script inspect-queues."""
+
+    def queueName(self, task_id):
+        return task_id.replace('-', '')
+
+    def runClearQueues(self, celery_config, task_ids):
+        """Invoke clear_queues() and catch the data written to stdout
+        and stderr.
+        """
+        # Simply calling the function clear_queues() from bin.clear_queues()
+        # leads to a one-time import of the celery config module; the
+        # config setting from the test that runs first would override
+        # any different configuration setting in a later test.
+        # Running the script in a subprocess avoids this problem.
+        queues = [self.queueName(task_id) for task_id in task_ids]
+        args = ['bin/clear-queues', '-c', celery_config] + queues
+        proc = subprocess.Popen(
+            args, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+            cwd=get_root())
+        stdout, stderr = proc.communicate()
+        return stdout, stderr
+
+    def invokeJob(self, celery_config, task, delay=1, job_args={}):
+        """Run the given task.
+
+        :return: The name of the result queue.
+        """
+        with tempdir() as temp_dir:
+            js = FileJobSource(temp_dir)
+            job = FileJob(js, 11, **job_args)
+            job.save()
+            task_info = task.apply_async(args=(11, ))
+            with celeryd(celery_config, temp_dir):
+                # Wait just long enough so that celeryd can start and
+                # process the job.
+                sleep(delay)
+            return task_info.task_id
+
+    def successMessage(self, task_id):
+        return (
+            "%s: {'status': 'SUCCESS', 'traceback': None, 'result': None, "
+            "'task_id': '%s'}\n" % (self.queueName(task_id), task_id))
+
+    def noQueueMessage(self, task_id):
+        return (
+            "NOT_FOUND - no queue '%s' in vhost '/'\n"
+            % self.queueName(task_id))
+
+    def test_clear_queues__result_not_consumed(self):
+        """When a Celery task is started so that a result is returned
+        but the result is not consumed, the related message can be
+        retrieved with clear_queues().
+        """
+        celery_config_jobrunner = 'lazr.jobrunner.tests.config1'
+        task_id = self.invokeJob(celery_config_jobrunner, RunFileJob)
+        # The script clear_queues does not have to use the same
+        # Celery configuration as the job runner: clear_config just
+        # needs to know how to connect to AMQP server.
+        clear_queue_config = 'lazr.jobrunner.tests.simple_config'
+        stdout, stderr = self.runClearQueues(clear_queue_config, [task_id])
+        self.assertEqual(self.successMessage(task_id), stdout)
+        self.assertEqual('', stderr)
+
+        # Reading a queue is destructive. An attempt to read again from
+        # a queue results in an error.
+        stdout, stderr = self.runClearQueues(clear_queue_config, [task_id])
+        self.assertEqual('', stdout)
+        self.assertEqual(self.noQueueMessage(task_id), stderr)
+
+    def test_clear_queues__two_queues(self):
+        """More than one queue can be inspected in one call of
+        clear_queue().
+        """
+        celery_config_jobrunner = 'lazr.jobrunner.tests.config1'
+        task_id_1 = self.invokeJob(celery_config_jobrunner, RunFileJob)
+        task_id_2 = self.invokeJob(celery_config_jobrunner, RunFileJob)
+        clear_queue_config = 'lazr.jobrunner.tests.simple_config'
+        stdout, stderr = self.runClearQueues(
+            clear_queue_config, [task_id_1, task_id_2])
+        expected_stdout = (
+            self.successMessage(task_id_1) + self.successMessage(task_id_2))
+        self.assertEqual(expected_stdout, stdout)
+        self.assertEqual('', stderr)
+
+    def test_clear_queues__task_without_result(self):
+        """A Celery task which was started so that no result is returned
+        does not write to a task queue.
+        """
+        celery_config_jobrunner = 'lazr.jobrunner.tests.config1'
+        task_id = self.invokeJob(celery_config_jobrunner, RunFileJobNoResult)
+        clear_queue_config = 'lazr.jobrunner.tests.simple_config'
+        stdout, stderr = self.runClearQueues(clear_queue_config, [task_id])
+        self.assertEqual('', stdout)
+        self.assertEqual(self.noQueueMessage(task_id), stderr)
+
+    def test_clear_queues__config_create_missing_queues_false(self):
+        """If CELERY_CREATE_MISSING_QUEUES is False, drain_queues()
+        must override this setting by creating the router instance
+        with the parameter create_missing=True. Otherwise,
+        app.amqp.Router() will fail with
+        'celery.exceptions.QueueNotFound: "Queue ... is not defined in
+        CELERY_QUEUES"'. Note that this does not mean that a non-existent
+        queue is actually created, see
+        assertEqual(self.noQueueMessage(task_id), stderr) below...
+        """
+        celery_config = (
+            'lazr.jobrunner.tests.config_do_not_create_missing_queues')
+        # A test isolation problem: Specifying a configuration module does
+        # not have the desired effect
+        task_id = 'this-queue-does-not-exist'
+        stdout, stderr = self.runClearQueues(celery_config, [task_id])
+        self.assertEqual('', stdout)
+        self.assertEqual(self.noQueueMessage(task_id), stderr)
